/*
	*
	* Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* at your option, any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

// Local Headers
#include "DesQActionBar.hpp"

const int DESQACTIONBAR_HEIGHT = 36;

DesQActionBar::DesQActionBar( QWidget *parent, Qt::Edge edge ) : QWidget( parent ) {

	mEdge = edge;

	createUI();
};

void DesQActionBar::createUI() {

	setFixedHeight( DESQACTIONBAR_HEIGHT );
	setContentsMargins( QMargins() );

	stack = new QStackedWidget();
	stack->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );

	QVBoxLayout *wLyt = new QVBoxLayout();
	wLyt->setContentsMargins( QMargins() );
	wLyt->setSpacing( 0 );

	wLyt->addStretch();
	wLyt->addWidget( stack );
	wLyt->addStretch();

	QWidget *base = new QWidget();
	base->setObjectName( "DesQActionBar" );

	base->setLayout( wLyt );

	QHBoxLayout *lyt = new QHBoxLayout();
	lyt->setContentsMargins( QMargins() );
	lyt->setSpacing( 0 );
	lyt->addWidget( base );

	setLayout( lyt );
};

int DesQActionBar::addAction( int page, QString name, QIcon icon, QString target, QString tooltip, bool checkable, bool checked ) {

	if ( page == layoutList.count() ) {
		QWidget *w = new QWidget();

		QHBoxLayout *lyt = new QHBoxLayout();
		lyt->setContentsMargins( QMargins() );
		lyt->setSpacing( 0 );
		lyt->addStretch();
		lyt->addStretch();

		layoutList << lyt;

		w->setLayout( lyt );
		stack->addWidget( w );
	}

	if ( not itemList.contains( page ) )
		itemList[ page ] = QList<DesQActionBarItem*>();

	DesQActionBarItem *item = new DesQActionBarItem( name, icon, target, tooltip, checkable, checked, this );

	layoutList.value( page )->insertWidget( layoutList.value( page )->count() - 1, item );
	itemList[ page ] << item;

	connect( item, SIGNAL( clicked( QString ) ), this, SIGNAL( action( QString ) ) );
	connect( item, SIGNAL( switchToPage( int ) ), this, SLOT( switchToPage( int ) ) );

	return itemList.value( page ).indexOf( item );
};

void DesQActionBar::setIcon( int page, int item, QIcon icon ) {

	itemList.value( page ).value( item )->setIcon( icon );
};

void DesQActionBar::setItemEnabled( int page, int item ) {

	itemList.value( page ).value( item )->setEnabled();
};

void DesQActionBar::setItemDisabled( int page, int item ) {

	itemList.value( page ).value( item )->setDisabled();
};

void DesQActionBar::addSingleSpace( int page ) {

	if ( page == layoutList.count() ) {
		QWidget *w = new QWidget();

		QHBoxLayout *lyt = new QHBoxLayout();
		lyt->setContentsMargins( QMargins() );
		lyt->setSpacing( 0 );
		lyt->addStretch();
		lyt->addStretch();

		layoutList << lyt;

		w->setLayout( lyt );
		stack->addWidget( w );
	}

	layoutList.value( page )->insertWidget( layoutList.value( page )->count() - 1, new DesQActionBarSpace( this ) );
};

void DesQActionBar::addStretch( int page ) {

	if ( page == layoutList.count() ) {
		QWidget *w = new QWidget();

		QHBoxLayout *lyt = new QHBoxLayout();
		lyt->setContentsMargins( QMargins() );
		lyt->setSpacing( 0 );
		lyt->addStretch();
		lyt->addStretch();

		layoutList << lyt;

		w->setLayout( lyt );
		stack->addWidget( w );
	}

	layoutList.value( page )->addStretch();
};

void DesQActionBar::addSeparator( int page ) {

	if ( page == layoutList.count() ) {
		QWidget *w = new QWidget();

		QHBoxLayout *lyt = new QHBoxLayout();
		lyt->setContentsMargins( QMargins() );
		lyt->setSpacing( 0 );
		lyt->addStretch();
		lyt->addStretch();

		layoutList << lyt;

		w->setLayout( lyt );
		stack->addWidget( w );
	}

	DesQActionBarSeparator *item = new DesQActionBarSeparator( mEdge, this );
	layoutList.value( page )->insertWidget( layoutList.value( page )->count() - 1, item );
};

void DesQActionBar::switchToPage( int page ) {

	if ( page >= stack->count() )
		return;

	stack->setCurrentIndex( page );
};

/*
	*
	* DesQActionBarItem - SidePanelItem class
	*
*/

DesQActionBarItem::DesQActionBarItem( QString name, QIcon icon, QString target, QString tooltip, bool checkable, bool checked, QWidget *parent ) : QWidget( parent ) {

	setFixedSize( QSize( 36, 36 ) );
	setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );

	mName = name;
	mIcon = icon;
	mTarget = target;

	setToolTip( tooltip );

	mHover = false;
	mPressed = false;

	mCheckable = checkable;
	mChecked = checked;

	setCursor( Qt::PointingHandCursor );
	setAcceptDrops( true );
};

void DesQActionBarItem::setIcon( QIcon icon ) {

	mIcon = icon;
	repaint();
};

void DesQActionBarItem::setEnabled() {

	QWidget::setEnabled( true );
};

void DesQActionBarItem::setDisabled() {

	QWidget::setDisabled( true );
};

void DesQActionBarItem::enterEvent( QEvent *eEvent ) {

	mHover = true;
	repaint();

	eEvent->accept();
};

void DesQActionBarItem::leaveEvent( QEvent *lEvent ) {

	mHover = false;
	repaint();

	lEvent->accept();
};

void DesQActionBarItem::mousePressEvent( QMouseEvent *mpEvent ) {

	if ( mpEvent->button() == Qt::LeftButton )
			mPressed = true;

	else if ( mpEvent->button() == Qt::RightButton ) {
	}

	repaint();
	mpEvent->accept();
};

void DesQActionBarItem::mouseReleaseEvent( QMouseEvent *mrEvent ) {

	if ( mrEvent->button() == Qt::LeftButton ) {

		mPressed = false;
		QStringList targets = mTarget.split( ";" );
		Q_FOREACH( QString target, targets ) {
			if ( target.startsWith( "#" ) )
				emit switchToPage( target.replace( "#", "" ).toUInt() );

			else
				emit clicked( target );
		}

		if ( mCheckable )
			mChecked = not mChecked;

		emit clicked();
	}

	repaint();
	mrEvent->accept();
};

void DesQActionBarItem::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	painter.setPen( Qt::NoPen );

	if ( not isEnabled() ) {
		painter.setBrush( palette().color( QPalette::Window ) );
		painter.drawRect( QRect( 0, 0, width(), height() ) );

		/* Paint Icon */
		QRect iconRect = QRect( 3, 3, 28, 28 );
		painter.drawPixmap( iconRect, mIcon.pixmap( 28, QIcon::Disabled ) );
		painter.end();

		return;
	}

	painter.save();
	if ( mPressed ) {

		painter.setBrush( palette().color( QPalette::Highlight ).darker( 120 ) );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}

	/* Hover and no highlight */
	else if ( mHover ) {

		painter.setBrush( palette().color( QPalette::Highlight ).lighter( 110 ) );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}

	else if ( mChecked ) {
		painter.setBrush( palette().color( QPalette::Highlight ).darker( 130 ) );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}

	/* Clear */
	else {

		painter.setBrush( Qt::transparent );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}
	painter.restore();

	/* Paint Icon */
	QRect iconRect = QRect( 3, 3, 28, 28 );
	painter.drawPixmap( iconRect, mIcon.pixmap( 28 ) );
	painter.end();

	pEvent->accept();
};

DesQActionBarSpace::DesQActionBarSpace( QWidget *parent ) : QWidget( parent ) {

	setFixedSize( QSize( 36, 36 ) );
	setDisabled( true );
};

DesQActionBarSeparator::DesQActionBarSeparator( Qt::Edge edge, QWidget *parent ) : QWidget( parent ) {

	switch( edge ) {
		case Qt::LeftEdge:
		case Qt::RightEdge:
			setFixedSize( QSize( 36, 1 ) );
			break;

		case Qt::TopEdge:
		case Qt::BottomEdge:
			setFixedSize( QSize( 1, 36 ) );
			break;
	}
};

void DesQActionBarSeparator::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	painter.setPen( Qt::gray );
	painter.drawRect( rect() );
	painter.end();

	pEvent->accept();
};
